#include <Wire.h> //This library allows you to communicate with I2C / TWI devices.
#include <WiFi.h>
#include <SPI.h>
#include<WString.h>

char ssid[] = "SSID";
char pass[] = "Password";

int status = WL_IDLE_STATUS;

WiFiClient client;
IPAddress server(130,206,83,2); 

int reading = 0;
boolean detected;
int car_out=0;

void setup()
{
  Wire.begin();                // join i2c bus for using HC SR501
  Serial.begin(9600);

  while (status != WL_CONNECTED) { 
    status = WiFi.begin(ssid, pass);
    delay(10000); 
  } 
  Serial.println("Wifi OK");
  detected =false;  // start serial communication at 9600bps
  attachInterrupt(0,trig, RISING);// 0-->pin2 1--Pin3
  delay(1000);
  updateContext(car_out);
}

void loop(){
  while(detected){
    reading=getDistance();        
    Serial.println(reading);   // print the readed measure
    if(reading < 100){
      detected=false;
      attachInterrupt(0,trig,RISING);
      Serial.println("There's a car leaving the parking");
      car_out+=1;
      updateContext(car_out);
      Serial.print("Cars out ");
      Serial.print(car_out);
    }
    delay(250);
  }

}

//FUNCTIONS

void trig(){
  if(!detected){
    detected=true;
    detachInterrupt(0);
  } 
}

int getDistance(){
  int dist=0;
  // step 1: instruct sensor to read echoes
  Wire.beginTransmission(112); 

  Wire.write(byte(0x00));       
  Wire.write(byte(0x51));       
  Wire.endTransmission();     
  //step 2: wait for readings to happen
  delay(70);                  
  // step 3: instruct sensor to return a particular echo reading
  Wire.beginTransmission(112); // transmit to device #112
  Wire.write(byte(0x02));      // sets register pointer to echo #1 register (0x02)
  Wire.endTransmission();      // stop transmitting
  // step 4: request reading from sensor
  Wire.requestFrom(112, 2);    // request 2 bytes from slave device #112
  // step 5: receive reading from sensor
  if(2 <= Wire.available()){   // if two bytes were received
    dist = Wire.read();  // receive high byte (overwrites previous reading)
    dist= dist << 8;    // shift high byte to be high 8 bits
    dist |= Wire.read(); // receive low byte as lower 8 bits
  }
  return dist;
}

void updateContext(int val){

  if (client.connect(server, 1026)) {
    Serial.println("connected to server");
    client.println("POST /NGSI10/updateContext HTTP/1.1");
    client.println("Host: 130.206.83.2:1026");
    client.println("User-Agent: Arduino/1.1");
    client.println("Connection: close");
    client.println("Content-Type: application/xml");
    client.print("Content-Length: ");
    client.println("460");
    client.println();
    client.println("<?xml version=\"1.0\" encoding=\"UFT-8\"?>");
    client.println("<updateContextRequest>");
    client.println("<contextElementList>");
    client.println("<contextElement>");
    client.println("<entityId type=\"UPCT:PARKING\" isPattern=\"false\">");
    client.println("<id>UPCT:PARKING:ED</id>");
    client.println("</entityId>");
    client.println("<contextAttributeList>");
    client.println("<contextAttribute>");
    client.println("<name>car_out</name>");
    client.println("<type>lot</type>");
    client.print("<contextValue>");
    client.print(val);
    client.println("</contextValue>");
    client.println("</contextAttribute>");
    client.println("</contextAttributeList>");
    client.println("</contextElement>");
    client.println("</contextElementList>");
    client.println("<updateAction>UPDATE</updateAction>");
    client.println("</updateContextRequest>");
    client.println();

    Serial.println("updateContext Done");
    while(!client.available());//Using this while we wait for and reply but it can have problems
                            //It's possible to use delay(750); instead,
  }
  else{
    Serial.println("Impossible to connect");
  }

  
  //Use when printing Http response is required
  //while (client.available()) {
  //char c = client.read();
  //Serial.write(c);           
  //}

  //if (!client.connected()) {
    Serial.println("disconnecting from server.");
    //Serial.println(out_car);
    client.stop();
  //}

}

int searchValue(String s,int i) {
  int beginning,ending;
  String val;
  beginning=s.indexOf('>');
  ending=s.indexOf('<', beginning+1);
  if(s.startsWith("contextValue",s.indexOf('<')+1)){
    val =s.substring(beginning+1,ending);
    return val.toInt();
  }
  else{
    return i;
  }
}


