#include <Sensirion.h>
#include <Wire.h> //This library allows you to communicate with I2C / TWI devices.
#include <WiFi.h>
#include <SPI.h>


char ssid[] = "SSID";
char pass[] = "Password";

int status = WL_IDLE_STATUS;

WiFiClient client;
IPAddress server(130,206,83,2); 

int red_led=8;
int green_led=9;

const uint8_t dataPin  =  6;
const uint8_t clockPin =  5;

float temperature;
float humidity;
float dewpoint;

Sensirion tempSensor = Sensirion(dataPin, clockPin);

int reading[] = {0,0,0};
int occupied = 0;
int update_time=10; //in x3sec ie --> update_time=3--> 3*3=9 sec
int count=0;
void setup()
{
  Wire.begin();                // join i2c bus for using HC SR501
  Serial.begin(9600);

  while (status != WL_CONNECTED) { 
    status = WiFi.begin(ssid, pass);
    delay(10000); 
  } 
  Serial.println("Wifi OK");

  pinMode(red_led, OUTPUT);
  pinMode(green_led, OUTPUT);
  
  updateLed(occupied);
}

void loop(){
  for(int i=0;i<3;i++){
    reading[i]=getDistance();
    Serial.println(reading[i]);
    delay(150);    
  }
  float average =(reading[0]+reading[1]+reading[2])/3;
  Serial.println(average);
  if(average<50){
   occupied=1;
  }else{occupied=0;}
  updateLed(occupied);
  tempSensor.measure(&temperature, &humidity, &dewpoint);
  Serial.print("Temperature: ");
  Serial.println(temperature);
  Serial.print("Humidity: ");
  Serial.println(humidity);
  Serial.print("occupied: ");
  Serial.println(occupied);
  if(count>=update_time){
  updateContext(temperature,humidity,occupied);
  count=0;
  }
  count++;
  delay(3000);
}

//FUNCTIONS

int getDistance(){
  int dist=0;
  // step 1: instruct sensor to read echoes
  Wire.beginTransmission(112); 

  Wire.write(byte(0x00));       
  Wire.write(byte(0x51));       
  Wire.endTransmission();     
  //step 2: wait for readings to happen
  delay(70);                  
  // step 3: instruct sensor to return a particular echo reading
  Wire.beginTransmission(112); // transmit to device #112
  Wire.write(byte(0x02));      // sets register pointer to echo #1 register (0x02)
  Wire.endTransmission();      // stop transmitting
  // step 4: request reading from sensor
  Wire.requestFrom(112, 2);    // request 2 bytes from slave device #112
  // step 5: receive reading from sensor
  if(2 <= Wire.available()){   // if two bytes were received
    dist = Wire.read();  // receive high byte (overwrites previous reading)
    dist= dist << 8;    // shift high byte to be high 8 bits
    dist |= Wire.read(); // receive low byte as lower 8 bits
  }
  return dist;
}

void updateLed(int val){
  if(val==0){
    digitalWrite(red_led, LOW);
    digitalWrite(green_led, HIGH);
  }
  if(val==1){
    digitalWrite(red_led, HIGH);
    digitalWrite(green_led, LOW);  
  } 
}

void updateContext(float val1,float val2,int val3){

  if (client.connect(server, 1026)) {
    Serial.println("connected to server");
    client.println("POST /NGSI10/updateContext HTTP/1.1");
    client.println("Host: 130.206.83.2:1026");
    client.println("User-Agent: Arduino/1.1");
    client.println("Connection: close");
    client.println("Content-Type: application/xml");
    client.print("Content-Length: ");
    client.println("710");
    client.println();
    client.println("<?xml version=\"1.0\" encoding=\"UFT-8\"?>");
    client.println("<updateContextRequest>");
    client.println("<contextElementList>");
    client.println("<contextElement>");
    client.println("<entityId type=\"UPCT:PARKING\" isPattern=\"false\">");
    client.println("<id>UPCT:PARKING:D1</id>");
    client.println("</entityId>");
    client.println("<contextAttributeList>");
    client.println("<contextAttribute>");
    client.println("<name>temperature</name>");
    client.println("<type>centigrade</type>");
    client.print("<contextValue>");
    client.print(val1);
    client.println("</contextValue>");
    client.println("</contextAttribute>");
    client.println("<contextAttribute>");
    client.println("<name>humidity</name>");
    client.println("<type>relative</type>");
    client.print("<contextValue>");
    client.print(val2);
    client.println("</contextValue>");
    client.println("</contextAttribute>");
    client.println("<contextAttribute>");
    client.println("<name>occupied</name>");
    client.println("<type>value</type>");
    client.print("<contextValue>");
    client.print(val3);
    client.println("</contextValue>");
    client.println("</contextAttribute>");
    client.println("</contextAttributeList>");
    client.println("</contextElement>");
    client.println("</contextElementList>");
    client.println("<updateAction>UPDATE</updateAction>");
    client.println("</updateContextRequest>");
    client.println();

    Serial.println("updateContext Done");

    while(!client.available());
    //Use when printing Http response is required
    /*while (client.available()) {
      char c = client.read();
      Serial.write(c);
    }*/
  }
  else{
    Serial.println("Impossible to connect");
  }

 

  if (!client.connected()) {
    Serial.println("disconnecting from server.");
    //Serial.println(out_car);
    client.stop();
  }

}
  





